__version__ = "0.0.11"


from sigla.lib.Node import Node
from sigla.lib.Processor import Processor
from sigla.lib.helpers.misc import cast_array


def process_node(root):
    processor = Processor()
    node = Node.from_xml(root)
    result = processor.process_node(node)
    result = cast_array(result)
    return result



from pathlib import Path


def ensure_dir(path):
    Path(path).mkdir(parents=True, exist_ok=True)


def ensure_parent_dir(path):
    Path(path) \
        .parent \
        .mkdir(parents=True, exist_ok=True)



import os

from sigla.lib.helpers.files import ensure_parent_dir


class SiglaFile:
    def __init__(self, content, path):
        self.content = content
        self.path = path

    def save(self, save_path=None):
        if save_path:
            path = os.path.join(os.getcwd(), save_path, self.path)
        else:
            path = os.path.join(os.getcwd(), self.path)

        ensure_parent_dir(path)

        with open(path, "w") as h:
            h.write(self.content)



from xml.etree import ElementTree as ET


class Node(object):
    attributes = {}
    meta = {}
    children = []

    def __init__(self, children=None, attributes=None, meta=None):
        if children is None:
            children = []
        if attributes is None:
            attributes = {}
        if meta is None:
            meta = {}

        self.attributes = attributes
        self.meta = meta
        self.children = children

    @classmethod
    def from_xml(cls, node: ET.Element):
        attributes = {}
        children = []
        for k, v in node.attrib.items():

            if k.endswith("-int"):
                attributes[k.replace("-int", "")] = int(v)
            else:
                attributes[k] = v
        for child in node:
            children.append(cls.from_xml(child))

        meta = {
            "tag": node.tag.split("-")[-1],
            "otag": node.tag,  # otag => original tag
        }

        return cls(children=children, attributes=attributes, meta=meta)

    def prettyprint(self, depth=0):
        sep = " " * depth * 4
        if len(self.children) > 0:
            child = (
                "\n"
                + "\n".join(
                    map(lambda x: x.prettyprint(depth=depth + 1), self.children)
                )
                + f"\n{sep}"
            )
        else:
            child = ""
        return f"""{sep}<{self.tag} {repr(self.attributes)}>{child}</{self.tag}>"""

    def __getattribute__(self, name):
        try:
            return object.__getattribute__(self, name)
        except AttributeError as e:
            if name in self.attributes.keys():
                return self.attributes[name]
            elif name in self.meta.keys():
                return self.meta[name]
            else:
                raise e

    def __iter__(self):
        return iter(self.children)



import click
from sigla import __version__
from sigla.lib.SnapshotCli import SnapshotCli, SNAPSHOTS_DIRECTORY
from sigla.lib.helpers.files import ensure_dir
from sigla.main import run

import logging

logging.basicConfig(
    level=logging.INFO,
    format="[%(levelname)s] %(message)s",
    handlers=[
        logging.FileHandler("debug.log"),
        # logging.StreamHandler()
    ]
)


# @click.option("--debug/--no-debug", default=False)
# click.echo('Debug mode is %s' % ('on' if debug else 'off'))
@click.group()
def cli():
    pass


@cli.command()  # @cli, not @click!
@click.argument("files", nargs=-1, type=click.Path(exists=True))
def render(files):
    for file in files:
        run(file)


@cli.command()
def version():
    """
    Prints the current version
    """
    click.echo(f"Version: {__version__}")


@cli.command()
@click.argument("files", nargs=-1, type=click.Path(exists=True))
def snapshot(files):
    """
    Makes snapshots
    """

    for file in files:
        snap = SnapshotCli(file)
        snap.make_snapshots()


@cli.command()
@click.argument("files", nargs=-1, type=click.Path(exists=True))
def verify(files):
    """
    Verify snapshots
    """

    for file in files:
        snap = SnapshotCli(file)
        snap.verify_snapshots()


if __name__ == "__main__":
    cli()



import json
from pathlib import Path

from sigla.lib.Node import Node
from sigla.lib.SiglaFile import SiglaFile
from sigla.lib.template.TemplateContext import TemplateContext
from sigla.lib.template.engines.njk import njk
from sigla.lib.helpers.files import ensure_parent_dir
from sigla.lib.helpers.misc import cast_array


def default_njk_template(dumped_context):
    return f"""

Available vars: {dumped_context}

Handle children:

{{{{ render(children) }}}}

{{% for child in children %}}
    {{{{ render(child) }}}}
{{% endfor %}}


"""


class Processor:
    ctx = None
    templates_directory = "./.sigla/templates"

    def __init__(self, ctx=None):
        if ctx is None:
            ctx = TemplateContext()
        self.ctx = ctx

    def process_file(self, node):

        if not node.name:
            raise Exception(f"# No name attached to the file element.")

        self.ctx.push_context(node)

        text = ""
        for child in node:
            text += self.process_node(child)

        self.ctx.pop_context()

        return SiglaFile(text, node.name)

    def process_node(self, node: Node):

        if node.tag == "root":
            process = self.process_node
            return list(map(process, node.children))

        if node.tag == "file":
            return self.process_file(node)

        else:
            # render a template
            return self.render_template(node.otag, node)

    def process_nodes_to_str(self, node):
        nodes = map(self.process_node, cast_array(node))
        return "".join(nodes)

    def render_template(self, name, node):
        create_missing_templates = True

        name = name.replace("-", "/")

        context = self.ctx.push_context(node)
        template_full_path = f"{self.templates_directory}/{name}.njk"
        dumped_context = json.dumps(context)
        dumped_context_keys = json.dumps(list(context.keys()) + ["children"])

        my_file = Path(template_full_path)
        if not my_file.exists() and create_missing_templates:
            ensure_parent_dir(template_full_path)
            with open(template_full_path, "w") as h:
                h.write(default_njk_template(dumped_context_keys))

        result = njk(
            template_full_path,
            **context,
            children=node.children,
            render=self.process_nodes_to_str,
            context=dumped_context,
        )

        self.ctx.pop_context()
        return result



import logging

from sigla import process_node
from sigla.lib.SiglaFile import SiglaFile
from sigla.lib.helpers.loaders import load_xml, load_string


def run(file=None, content=None):
    if file:
        root = load_xml(file)
    elif content:
        root = load_string(content)
    else:
        raise Exception("You need to prove a file or some content")

    result = process_node(root)
    for r in result:
        if type(r) == SiglaFile:
            logging.info(f"[Render] Saving generated file to {r.path}")
            r.save()
        elif type(r) == str:
            print(r)
        else:
            raise NotImplementedError(f"No final handling implemented for {type(r)}")


import pydash as _
from jinja2 import FileSystemLoader, Environment


def as_kwargs_filter(obj):
    kwargs = []
    for k, v in obj.items():
        if type(v) == int:
            kwargs.append(f"{k}={v}")
        else:
            kwargs.append(f'{k}="{v}"')
    return ", ".join(kwargs)


def map_get_filter(arr, key):
    return [_.get(o, key) for o in arr]


def without_filter(obj, *args):
    result = {}
    for k, v in obj.items():
        if k not in args:
            result[k] = v
    return result


def njk(template_full_path, **kwargs):
    with open(template_full_path, "r") as h:
        file_loader = FileSystemLoader(".")
        env = Environment(loader=file_loader)
        env.filters["without"] = without_filter
        env.filters["as_kwargs"] = as_kwargs_filter
        env.filters["map_get"] = map_get_filter

        env.filters["get_nested"] = lambda arr, field: map(
            lambda el: _.get(el, field), arr
        )

        env.filters["flatten"] = _.flatten

        # template = Template(h.read())
        template = env.get_template(template_full_path)

        return template.render(**kwargs)
